package com.muyu.product.domain.model;

import com.muyu.common.core.annotation.Excel;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.DecimalMin;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;

/**
 * @author DongZl
 * @description: 商品Sku
 * @Date 2024-3-22 上午 10:54
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@ApiModel(value = "商品SKU模型", description = "商品SKU信息")
public class ProductSkuModel {

    /** SKU编码 */
    @NotBlank(message = "SKU编码不能为空")
    @Excel(name = "SKU编码")
    @ApiModelProperty(name = "sku", value = "SKU编码", required = true, example = "SP123456")
    private String sku;

    /** 商品库存 */
    @NotNull(message = "商品库存不能为空")
    @Min(value = 0, message = "商品库存不能小于0")
    @Excel(name = "商品库存")
    @ApiModelProperty(name = "stock", value = "商品库存", required = true, example = "100")
    private Long stock;

    /** 销售价格 */
    @NotNull(message = "销售价格不能为空")
    @DecimalMin(value = "0.01", message = "销售价格必须大于0")
    @Excel(name = "销售价格")
    @ApiModelProperty(name = "price", value = "销售价格", required = true, example = "99.99")
    private BigDecimal price;

    /** 市场价格 */
    @Excel(name = "市场价格")
    @ApiModelProperty(name = "marketPrice", value = "市场价格", example = "129.99")
    private BigDecimal marketPrice;

    /** 成本价格 */
    @Excel(name = "成本价格")
    @ApiModelProperty(name = "costPrice", value = "成本价格", example = "59.99")
    private BigDecimal costPrice;

    /** 规格图片 */
    @NotBlank(message = "规格图片不能为空")
    @Excel(name = "规格图片")
    @ApiModelProperty(name = "image", value = "规格图片", required = true)
    private String image;

    /** 规格值JSON */
    @ApiModelProperty(name = "specValues", value = "规格值JSON", example = "{\"颜色\":\"红色\",\"尺寸\":\"XL\"}")
    private String specValues;

    /** 商品条形码 */
    @Excel(name = "商品条形码")
    @ApiModelProperty(name = "barcode", value = "商品条形码")
    private String barcode;

    /** 预警库存 */
    @Min(value = 0, message = "预警库存不能小于0")
    @Excel(name = "预警库存")
    @ApiModelProperty(name = "lowStock", value = "预警库存", example = "10")
    private Integer lowStock;

    /** 锁定库存 */
    @Excel(name = "锁定库存")
    @ApiModelProperty(name = "lockStock", value = "锁定库存", example = "0")
    private Integer lockStock;

    /** 销量 */
    @Excel(name = "销量")
    @ApiModelProperty(name = "sales", value = "销量", example = "0")
    private Integer sales;

    /** 状态（0-禁用 1-启用） */
    @Excel(name = "状态", readConverterExp = "0=禁用,1=启用")
    @ApiModelProperty(name = "status", value = "状态（0-禁用 1-启用）", example = "1")
    private String status;

    /** SKU名称 */
    @Excel(name = "SKU名称")
    @ApiModelProperty(name = "skuName", value = "SKU名称")
    private String skuName;

    /** 重量(克) */
    @Excel(name = "重量(克)")
    @ApiModelProperty(name = "weight", value = "重量(克)")
    private Double weight;

    /** 体积(cm³) */
    @Excel(name = "体积(cm³)")
    @ApiModelProperty(name = "volume", value = "体积(cm³)")
    private Double volume;

    /** 排序号 */
    @Excel(name = "排序号")
    @ApiModelProperty(name = "sort", value = "排序号", example = "1")
    private Integer sort;
}
