package com.muyu.product.service.impl;

import com.alibaba.nacos.client.naming.utils.CollectionUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.product.domain.*;
import com.muyu.product.domain.base.CategoryBase;
import com.muyu.product.domain.model.CategoryInfoSaveModel;
import com.muyu.product.domain.model.TemplateAttributeGroupModel;
import com.muyu.product.domain.model.TemplateAttributeModel;
import com.muyu.product.domain.resp.CategoryCommonElementResp;
import com.muyu.product.domain.resp.CategoryInfoDetailResp;
import com.muyu.product.domain.resp.CategoryParentCommonElementResp;
import com.muyu.product.mapper.CategoryInfoMapper;
import com.muyu.product.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 品类信息Service业务层处理
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Slf4j
@Service
public class CategoryInfoServiceImpl extends ServiceImpl<CategoryInfoMapper, CategoryInfo> implements CategoryInfoService {


    @Autowired
    private AsCategoryAttributeService asCategoryAttributeService;

    @Autowired
    private AsCategoryAttributeGroupService asCategoryAttributeGroupService;

    @Autowired
    private AsCategoryBrandService asCategoryBrandService;

    @Autowired
    private AsAttributeGroupService asAttributeGroupService;

    @Autowired
    private AttributeInfoService attributeInfoService;

    @Autowired
    private BrandInfoService brandInfoService;

    @Autowired
    private AttributeGroupService attributeGroupService;

    @Lazy
    @Autowired
    private ProjectInfoService projectInfoService;

    @Autowired
    private AsProductAttributeInfoService asProductAttributeInfoService;

    @Override
    public List<CategoryInfoDetailResp> listWithParent(CategoryInfo categoryInfo) {
        // 1. 添加空值检查, 改为使用Optional
        categoryInfo = Optional.ofNullable(categoryInfo).orElse(new CategoryInfo());

        // 2. 查询品类列表
        LambdaQueryWrapper<CategoryInfo> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(categoryInfo.getName())) {
            queryWrapper.like(CategoryInfo::getName, categoryInfo.getName());
        }
        // ... 其他查询条件

        List<CategoryInfo> categoryList = list(queryWrapper);
        if (CollectionUtils.isEmpty(categoryList)) {
            return new ArrayList<>();
        }

        // 3. 获取所有父级ID时添加空值检查
        Set<Long> parentIds = categoryList.stream()
                .map(CategoryInfo::getParentId)
                .filter(Objects::nonNull)  // 添加非空过滤
                .filter(id -> id != 0)
                .collect(Collectors.toSet());

        // 4. 批量查询父级品类
        Map<Long, String> parentNameMap = new HashMap<>();
        if (!parentIds.isEmpty()) {
            List<CategoryInfo> parentCategories = listByIds(parentIds);
            parentNameMap = parentCategories.stream()
                    .collect(Collectors.toMap(
                            CategoryInfo::getId,
                            CategoryInfo::getName,
                            (k1, k2) -> k1
                    ));
        }

        // 5. 转换为响应对象
        Map<Long, String> finalParentNameMap = parentNameMap;
        return categoryList.stream()
                .map(category -> {
                    String parentName = null;
                    if (category.getParentId() != null && category.getParentId() != 0) {
                        parentName = finalParentNameMap.get(category.getParentId());
                    }
                    return CategoryInfoDetailResp.of(category, parentName);
                })
                .collect(Collectors.toList());
    }

    /**
     * 查询品类信息列表
     *
     * @param categoryInfo 品类信息
     *
     * @return 品类信息
     */
//    @Override
//    public List<CategoryInfo> list(CategoryInfo categoryInfo) {
//        // 1. 查询品类列表
//        LambdaQueryWrapper<CategoryInfo> queryWrapper = new LambdaQueryWrapper<>();
//        if (ObjUtils.notNull(categoryInfo.getName())) {
//            queryWrapper.like(CategoryInfo::getName, categoryInfo.getName());
//        }
//        if (ObjUtils.notNull(categoryInfo.getImage())) {
//            queryWrapper.eq(CategoryInfo::getImage, categoryInfo.getImage());
//        }
//        if (ObjUtils.notNull(categoryInfo.getParentId())) {
//            queryWrapper.eq(CategoryInfo::getParentId, categoryInfo.getParentId());
//        }
//        if (ObjUtils.notNull(categoryInfo.getStart())) {
//            queryWrapper.eq(CategoryInfo::getStart, categoryInfo.getStart());
//        }
//        if (ObjUtils.notNull(categoryInfo.getIntroduction())) {
//            queryWrapper.eq(CategoryInfo::getIntroduction, categoryInfo.getIntroduction());
//        }
//
//        List<CategoryInfo> categoryList = list(queryWrapper);
//
//        // 2. 获取所有父级ID
//        Set<Long> parentIds = categoryList.stream()
//                .map(CategoryInfo::getParentId)
//                .filter(id -> id != 0)
//                .collect(Collectors.toSet());
//
//        // 3. 批量查询父级品类
//        Map<Long, String> parentNameMap = new HashMap<>();
//        if (!parentIds.isEmpty()) {
//            List<CategoryInfo> parentCategories = listByIds(parentIds);
//            parentNameMap = parentCategories.stream()
//                    .collect(Collectors.toMap(
//                            CategoryInfo::getId,
//                            CategoryInfo::getName
//                    ));
//        }
//
//        // 4. 转换为响应对象
//        Map<Long, String> finalParentNameMap = parentNameMap;
//        return categoryList.stream()
//                .map(category -> {
//                    String parentName = category.getParentId() != 0 ?
//                            finalParentNameMap.get(category.getParentId()) : null;
//                    return CategoryInfoDetailResp.of(category, parentName);
//                })
//                .collect(Collectors.toList());
//    }

    /**
     * 保存商品品类
     * @param categoryInfoSaveModel 保存品类模型
     * @return 是否成功
     */
    @Override
    @Transactional
    public boolean save (CategoryInfoSaveModel categoryInfoSaveModel) {
        CategoryInfo categoryInfo = CategoryInfo.saveModelBuild(categoryInfoSaveModel);
        boolean save = this.save(categoryInfo);
        Long categoryInfoId = categoryInfo.getId();
        List<Long> attributeIdList = categoryInfoSaveModel.getAttributeIdList();
        if (attributeIdList != null && !attributeIdList.isEmpty()) {
            asCategoryAttributeService.saveBatch(
                    attributeIdList.stream()
                            .map(attributeId -> AsCategoryAttribute.categoryBuild(categoryInfoId, attributeId))
                            .toList()
            );
        }
        List<Long> attributeGroupIdList = categoryInfoSaveModel.getAttributeGroupIdList();
        if (attributeGroupIdList != null && !attributeGroupIdList.isEmpty()) {
            asCategoryAttributeGroupService.saveBatch(
                    attributeGroupIdList.stream()
                            .map(attributeGroupId -> AsCategoryAttributeGroup.categoryBuild(categoryInfoId, attributeGroupId))
                            .toList()
            );
        }
        List<Long> brandIdList = categoryInfoSaveModel.getBrandIdList();
        if (brandIdList != null && !brandIdList.isEmpty()) {
            asCategoryBrandService.saveBatch(
                    brandIdList.stream()
                            .map(brandId -> AsCategoryBrand.categoryBuild(categoryInfoId, brandId))
                            .toList()
            );
        }
        return save;
    }

    /**
     * 通过品类ID获取父级以上的属性组集合
     *
     * @param categoryId 品类ID
     *
     * @return 父级以上的属性组集合
     */
    @Override
    public List<AttributeGroup> getAttributeGroup (Long categoryId) {
        // 1. 空值检查
        if (categoryId == null) {
            return new ArrayList<>();
        }

        List<AttributeGroup> attributeGroupList = new ArrayList<>();
        CategoryInfo categoryInfo = this.getById(categoryId);

        // 2. 添加空值检查
        if (categoryInfo == null) {
            return attributeGroupList;
        }

        // 3. 查询当前品类关联的属性组
        LambdaQueryWrapper<AsCategoryAttributeGroup> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AsCategoryAttributeGroup::getCategoryId, categoryId);
        List<AsCategoryAttributeGroup> asCategoryAttributeGroupList = asCategoryAttributeGroupService.list(queryWrapper);

        // 4. 获取并添加当前品类的属性组
        if (asCategoryAttributeGroupList != null && !asCategoryAttributeGroupList.isEmpty()) {
            List<Long> attributeGroupIdList = asCategoryAttributeGroupList.stream()
                    .map(AsCategoryAttributeGroup::getAttributeGroupId)
                    .collect(Collectors.toList());
            attributeGroupList.addAll(attributeGroupService.listByIds(attributeGroupIdList));
        }

        // 5. 递归获取父级品类的属性组
        CategoryInfo categoryInfo1 = this.getById(categoryId);
        if (categoryInfo1.getParentId() != 0) {
            if (attributeGroupList.isEmpty()) {
                // 如果当前没有属性组，直接添加父级的所有属性组
                attributeGroupList.addAll(getAttributeGroup(categoryInfo1.getParentId()));
            } else {
                // 如果当前有属性组，只添加不重复的父级属性组
                List<AttributeGroup> attributeGroups = getAttributeGroup(categoryInfo1.getParentId());
                attributeGroups.forEach(attributeGroup -> {
                    if (!attributeGroupList.contains(attributeGroup)) {
                        attributeGroupList.add(attributeGroup);
                    }
                });
            }
        }

        return attributeGroupList;
    }

    /**
     * 通过品类ID获取父级以上的品牌集合
     *
     * @param categoryId 品类ID
     *
     * @return 父级以上的品牌集合
     */
    @Override
    public List<BrandInfo> getBrand (Long categoryId) {
        List<BrandInfo> brandInfoList = new ArrayList<>();
        LambdaQueryWrapper<AsCategoryBrand> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AsCategoryBrand::getCategoryId, categoryId);
        List<AsCategoryBrand> asCategoryBrandList = asCategoryBrandService.list(queryWrapper);
        if (asCategoryBrandList != null && !asCategoryBrandList.isEmpty()) {
            List<Long> brandIdList = asCategoryBrandList.stream()
                    .map(AsCategoryBrand::getBrandId)
                    .toList();
            brandInfoList.addAll(brandInfoService.listByIds(brandIdList));
        }
        CategoryInfo categoryInfo = this.getById(categoryId);
        if (categoryInfo.getParentId() != 0) {
            if (brandInfoList.isEmpty()) {
                brandInfoList.addAll(getBrand(categoryInfo.getParentId()));
            } else {
                List<BrandInfo> brandInfos = getBrand(categoryInfo.getParentId());
                brandInfos.forEach(brandInfo -> {
                    if (!brandInfoList.contains(brandInfo)) {
                        brandInfoList.add(brandInfo);
                    }
                });
            }

        }
        return brandInfoList;
    }

    /**
     * 通过品类ID获取父级以上的属性集合
     *
     * @param categoryId 品类ID
     *
     * @return 父级以上的属性集合
     */
    @Override
    public List<AttributeInfo> getAttribute (Long categoryId) {
        // 1. 入参校验
        if (categoryId == null || categoryId == 0) {
            return new ArrayList<>();
        }

        List<AttributeInfo> attributeInfoList = new ArrayList<>();

        // 2. 获取当前分类信息，使用Optional安全处理
        CategoryInfo categoryInfo = this.getById(categoryId);
        if (categoryInfo == null) {
            return attributeInfoList;
        }

        // 3. 查询当前分类关联的属性
        LambdaQueryWrapper<AsCategoryAttribute> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AsCategoryAttribute::getCategoryId, categoryId);
        List<AsCategoryAttribute> asCategoryAttributeList = asCategoryAttributeService.list(queryWrapper);

        // 4. 获取当前分类的属性
        if (CollectionUtils.isEmpty(asCategoryAttributeList)) {
            List<Long> attributeIdList = asCategoryAttributeList.stream()
                    .map(AsCategoryAttribute::getAttributeId)
                    .collect(Collectors.toList());
            if (CollectionUtils.isEmpty(attributeIdList)) {
                attributeInfoList.addAll(attributeInfoService.listByIds(attributeIdList));
            }
        }

        // 5. 处理父级属性，使用安全的parentId获取方式
        Long parentId = categoryInfo.getSafeParentId(); // 使用我们之前添加的安全获取方法
        if (parentId != 0) {
            CategoryInfo parentCategory = this.getById(parentId);
            if (parentCategory != null) {
                if (attributeInfoList.isEmpty()) {
                    // 如果当前没有属性，直接添加父级的所有属性
                    attributeInfoList.addAll(getAttribute(parentId));
                } else {
                    // 如果当前有属性，只添加不重复的父级属性
                    List<AttributeInfo> parentAttributes = getAttribute(parentId);
                    if (CollectionUtils.isEmpty(parentAttributes)) {
                        for (AttributeInfo attributeInfo : parentAttributes) {
                            if (!attributeInfoList.contains(attributeInfo)) {
                                attributeInfoList.add(attributeInfo);
                            }
                        }
                    }
                }
            }
        }

        return attributeInfoList;
    }

    @Override
    public <T, AS> List<T> getCommon (Long categoryId, IService<AS> iService, IService<T> bsiService) {
        List<T> list = new ArrayList();
        QueryWrapper<AS> asQueryWrapper = new QueryWrapper<>();
        asQueryWrapper.eq("category_id", categoryId);
        List<AS> asList = iService.list(asQueryWrapper);
        if (asList != null && !asList.isEmpty()) {
            List<Long> baseIdList = asList.stream()
                    .map(as -> {
                        if (as instanceof CategoryBase categoryBase) {
                            return categoryBase.getBaseId();
                        }
                        return null;
                    })
                    .toList();
            list.addAll(bsiService.listByIds(baseIdList));
        }
        CategoryInfo categoryInfo = this.getById(categoryId);
        if (categoryInfo.getParentId() != 0) {
            if (list.isEmpty()) {
                list.addAll(getCommon(categoryInfo.getParentId(), iService, bsiService));
            } else {
                List<T> commonList = getCommon(categoryInfo.getParentId(), iService, bsiService);
                commonList.forEach(common -> {
                    if (!list.contains(common)) {
                        list.add(common);
                    }
                });
            }
        }
        return list;
    }

    /**
     * 通过品类ID获取父级以上的属性、属性组、品牌集合
     *
     * @param categoryId 品类ID
     *
     * @return 父级以上的属性、属性组、品牌集合
     */
    @Override
    public CategoryParentCommonElementResp parentCommonElement (Long categoryId) {
        return CategoryParentCommonElementResp.builder()
                .attributeInfoList(getCommon(categoryId, asCategoryAttributeService, attributeInfoService))
                .attributeGroupList(getCommon(categoryId, asCategoryAttributeGroupService, attributeGroupService))
                .brandInfoList(getCommon(categoryId, asCategoryBrandService, brandInfoService))
                .build();
    }

    private void getParentIdListByCateGoryId(List<Long> parentIdList, Long cateGoryId){
        if (cateGoryId.equals(0L)){
            return;
        }
        CategoryInfo categoryInfo = this.getById(cateGoryId);
        parentIdList.add(categoryInfo.getId());
        getParentIdListByCateGoryId(parentIdList, categoryInfo.getParentId());
    }

    /**
     * 通过品类ID获取品类共有属性
     *
     * @param cateGoryId 品类ID
     *
     * @return 品类共有属性
     */
    @Override
    public CategoryCommonElementResp getTemplateAttributeByCateGoryId (Long cateGoryId) {
        List<Long> cateGoryIdList = new ArrayList<>();
        getParentIdListByCateGoryId(cateGoryIdList, cateGoryId);
        // 取出和品类相关联的属性组关系  - 中间表
        LambdaQueryWrapper<AsCategoryAttributeGroup> asCategoryAttributeGroupLambdaQueryWrapper = new LambdaQueryWrapper<>(){{
            in(AsCategoryAttributeGroup::getCategoryId, cateGoryIdList);}};
        List<AsCategoryAttributeGroup> categoryAttributeGroupList = asCategoryAttributeGroupService.list(asCategoryAttributeGroupLambdaQueryWrapper);
        List<TemplateAttributeGroupModel>  attributeGroupModelList = categoryAttributeGroupList.stream()
                .map(AsCategoryAttributeGroup::getAttributeGroupId).distinct().map(attributeGroupId -> TemplateAttributeGroupModel.attributeGroupBuild(
                        attributeGroupService.getById(attributeGroupId),
                        applyAttributeGroupId -> {
                            LambdaQueryWrapper<AsAttributeGroup> asAttributeGroupQueryWrapper = new LambdaQueryWrapper<>() {{
                                eq(AsAttributeGroup::getGroupId, applyAttributeGroupId);
                            }};
                            List<Long> attributeIdList = asAttributeGroupService.list(asAttributeGroupQueryWrapper).stream()
                                    .map(AsAttributeGroup::getAttributeId)
                                    .toList();
                            if (attributeIdList.isEmpty()) {
                                return new ArrayList<>();
                            }
                            return attributeInfoService.listByIds(attributeIdList).stream()
                                    .map(AttributeInfo::buildTemplateModel)
                                    .toList();
                        }
                ))
                .filter(TemplateAttributeGroupModel::isEffective)
                .toList();
        // 查重集合
        Set<Long> attributeIdSet = new HashSet<>();
        // 获取组内所有的属性Id
        if (!attributeGroupModelList.isEmpty()){
            attributeIdSet.addAll(
                    attributeGroupModelList.stream()
                            .flatMap((Function<TemplateAttributeGroupModel, Stream<TemplateAttributeModel>>) templateAttributeGroupModel -> templateAttributeGroupModel.getAttributeList().stream())
                            .map(TemplateAttributeModel::getId)
                            .toList()
            );
        }
        List<TemplateAttributeModel> templateAttributeModelList = new ArrayList<>();
        LambdaQueryWrapper<AsCategoryAttribute> categoryAttributeQueryWrapper = new LambdaQueryWrapper<>();
        categoryAttributeQueryWrapper.in(AsCategoryAttribute::getCategoryId, cateGoryIdList);
        List<AsCategoryAttribute> asCategoryAttributeList = asCategoryAttributeService.list(categoryAttributeQueryWrapper);
        if (asCategoryAttributeList != null && !asCategoryAttributeList.isEmpty()){
            List<Long> templateAttributeIdList = asCategoryAttributeList.stream()
                    .map(AsCategoryAttribute::getAttributeId)
                    .filter(templateAttributeId -> !attributeIdSet.contains(templateAttributeId))
                    .toList();
            templateAttributeModelList = attributeInfoService.listByIds(templateAttributeIdList).stream()
                    .map(AttributeInfo::buildTemplateModel)
                    .toList();
        }

        List<TemplateAttributeModel> attributeModelList = new ArrayList<>();
        if (!templateAttributeModelList.isEmpty()){
            attributeIdSet.addAll(
                    templateAttributeModelList.stream().map(TemplateAttributeModel::getId).toList()
            );
        }


        LambdaQueryWrapper<AttributeInfo> attributeInfoLambdaQueryWrapper = new LambdaQueryWrapper<>();
        attributeInfoLambdaQueryWrapper.notIn(!attributeIdSet.isEmpty(), AttributeInfo::getId, attributeIdSet);
        attributeModelList = attributeInfoService.list(attributeInfoLambdaQueryWrapper).stream().map(AttributeInfo::buildTemplateModel).toList();



        attributeGroupModelList.forEach(TemplateAttributeGroupModel->{
            TemplateAttributeGroupModel.getAttributeList().forEach(TemplateAttributeModel->{
                Long id = TemplateAttributeModel.getId();
                LambdaQueryWrapper<AsProductAttributeInfo> queryWrapper = new LambdaQueryWrapper<>();
                queryWrapper.eq(AsProductAttributeInfo::getAttributeId,id);
                List<AsProductAttributeInfo> list = this.asProductAttributeInfoService.list(queryWrapper);
                list.forEach(AsProductAttributeInfo->{
                    TemplateAttributeModel.setValue(AsProductAttributeInfo.getValue());
                });
            });
        });

        templateAttributeModelList.forEach(TemplateAttributeModel ->{
            Long id = TemplateAttributeModel.getId();
            LambdaQueryWrapper<AsProductAttributeInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(AsProductAttributeInfo::getAttributeId,id);
            List<AsProductAttributeInfo> list = this.asProductAttributeInfoService.list(lambdaQueryWrapper);
            list.forEach(asProductAttributeInfo -> {
                TemplateAttributeModel.setValue(asProductAttributeInfo.getValue());
            });
        });

        attributeModelList.forEach(templateAttributeModel -> {
            Long id = templateAttributeModel.getId();
            LambdaQueryWrapper<AsProductAttributeInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(AsProductAttributeInfo::getAttributeId,id);
            List<AsProductAttributeInfo> list = this.asProductAttributeInfoService.list(lambdaQueryWrapper);
            list.forEach(asProductAttributeInfo -> {
                templateAttributeModel.setValue(asProductAttributeInfo.getValue());
            });
        });
        return CategoryCommonElementResp.builder()
                .templateAttributeGroupList(attributeGroupModelList)
                .templateAttributeList(templateAttributeModelList)
                .attributeList(attributeModelList)
                .build();
    }

    @Override
    public CategoryInfoDetailResp getDetailById(Long id) {
        // 获取基本信息
        CategoryInfo categoryInfo = this.getById(id);
        if (categoryInfo == null) {
            return null;
        }

        CategoryInfoDetailResp resp = CategoryInfoDetailResp.of(categoryInfo);

        // 获取关联的属性ID列表
        LambdaQueryWrapper<AsCategoryAttribute> attrWrapper = new LambdaQueryWrapper<>();
        attrWrapper.eq(AsCategoryAttribute::getCategoryId, id);
        List<AsCategoryAttribute> categoryAttributes = asCategoryAttributeService.list(attrWrapper);
        List<Long> attributeIds = categoryAttributes.stream()
                .map(AsCategoryAttribute::getAttributeId)
                .collect(Collectors.toList());
        resp.setAttributeIdList(attributeIds);

        // 获取关联的属性组ID列表
        LambdaQueryWrapper<AsCategoryAttributeGroup> groupWrapper = new LambdaQueryWrapper<>();
        groupWrapper.eq(AsCategoryAttributeGroup::getCategoryId, id);
        List<AsCategoryAttributeGroup> categoryGroups = asCategoryAttributeGroupService.list(groupWrapper);
        List<Long> groupIds = categoryGroups.stream()
                .map(AsCategoryAttributeGroup::getAttributeGroupId)
                .collect(Collectors.toList());
        resp.setAttributeGroupIdList(groupIds);

        // 获取关联的品牌ID列表
        LambdaQueryWrapper<AsCategoryBrand> brandWrapper = new LambdaQueryWrapper<>();
        brandWrapper.eq(AsCategoryBrand::getCategoryId, id);
        List<AsCategoryBrand> categoryBrands = asCategoryBrandService.list(brandWrapper);
        List<Long> brandIds = categoryBrands.stream()
                .map(AsCategoryBrand::getBrandId)
                .collect(Collectors.toList());
        resp.setBrandIdList(brandIds);

        return resp;
    }



    @Override
    @Transactional
    public boolean updateCategoryInfo(Long id, CategoryInfoSaveModel updateModel) {
        // 更新基本信息
        CategoryInfo categoryInfo = CategoryInfo.saveModelBuild(updateModel);
        categoryInfo.setId(id);
        this.updateById(categoryInfo);

        // 更新关联属性
        updateAssociations(id,
                updateModel.getAttributeIdList(),
                updateModel.getAttributeGroupIdList(),
                updateModel.getBrandIdList()
        );

        return true;
    }

    @Override
    public boolean removeCategories(List<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return false;
        }
        // 删除品类关联数据
        for (Long categoryId : ids) {
            //检查是否有子类目
            LambdaQueryWrapper<CategoryInfo> childWrapper = new LambdaQueryWrapper<>();
            childWrapper.eq(CategoryInfo::getParentId,categoryId);
            long count = this.count(childWrapper);
            if (count > 0){
                CategoryInfo category = this.getById(categoryId);
                throw new RuntimeException("品类【"+category.getName()+"】存在子类目 ，不能删除");
            }
            //检查最后一级是否关联商品
            LambdaQueryWrapper<ProjectInfo> projectWrapper = new LambdaQueryWrapper<>();
            projectWrapper.eq(ProjectInfo::getMianType,categoryId)
                    .or()
                    .eq(ProjectInfo::getParentType,categoryId)
                    .or()
                    .eq(ProjectInfo::getType,categoryId);
            long projectCount = projectInfoService.count(projectWrapper);
            if (projectCount > 0){
                CategoryInfo category = this.getById(categoryId);
                throw new RuntimeException("品类【"+category.getName()+"】已关联项目，不能删除");
            }


            // 删除品类-属性关联
            LambdaQueryWrapper<AsCategoryAttribute> attrWrapper = new LambdaQueryWrapper<>();
            attrWrapper.eq(AsCategoryAttribute::getCategoryId, categoryId);
            asCategoryAttributeService.remove(attrWrapper);

            // 删除品类-属性组关联
            LambdaQueryWrapper<AsCategoryAttributeGroup> groupWrapper = new LambdaQueryWrapper<>();
            groupWrapper.eq(AsCategoryAttributeGroup::getCategoryId, categoryId);
            asCategoryAttributeGroupService.remove(groupWrapper);

            // 删除品类-品牌关联
            LambdaQueryWrapper<AsCategoryBrand> brandWrapper = new LambdaQueryWrapper<>();
            brandWrapper.eq(AsCategoryBrand::getCategoryId, categoryId);
            asCategoryBrandService.remove(brandWrapper);
        }

        // 删除品类
        return this.removeBatchByIds(ids);
    }
    private void updateAssociations(Long categoryId,
                                    List<Long> attributeIds,
                                    List<Long> groupIds,
                                    List<Long> brandIds) {
        // 删除原有关联
        asCategoryAttributeService.remove(new LambdaQueryWrapper<AsCategoryAttribute>()
                .eq(AsCategoryAttribute::getCategoryId, categoryId));
        asCategoryAttributeGroupService.remove(new LambdaQueryWrapper<AsCategoryAttributeGroup>()
                .eq(AsCategoryAttributeGroup::getCategoryId, categoryId));
        asCategoryBrandService.remove(new LambdaQueryWrapper<AsCategoryBrand>()
                .eq(AsCategoryBrand::getCategoryId, categoryId));

        // 保存新关联
        if (!CollectionUtils.isEmpty(attributeIds)) {
            asCategoryAttributeService.saveBatch(attributeIds.stream()
                    .map(attrId -> AsCategoryAttribute.categoryBuild(categoryId, attrId))
                    .collect(Collectors.toList()));
        }

        if (!CollectionUtils.isEmpty(groupIds)) {
            asCategoryAttributeGroupService.saveBatch(groupIds.stream()
                    .map(groupId -> AsCategoryAttributeGroup.categoryBuild(categoryId, groupId))
                    .collect(Collectors.toList()));
        }

        if (!CollectionUtils.isEmpty(brandIds)) {
            asCategoryBrandService.saveBatch(brandIds.stream()
                    .map(brandId -> AsCategoryBrand.categoryBuild(categoryId, brandId))
                    .collect(Collectors.toList()));
        }
    }

}
