package com.muyu.product.service.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.alibaba.nacos.common.utils.CollectionUtils;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.product.domain.*;
import com.muyu.product.domain.model.*;
import com.muyu.product.domain.req.ProjectInfoEditReq;
import com.muyu.product.domain.req.ProjectInfoSaveReq;
import com.muyu.product.domain.resp.CategoryCommonElementResp;
import com.muyu.product.domain.resp.ProjectDetailResp;
import com.muyu.product.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.product.mapper.ProjectInfoMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.transaction.annotation.Transactional;

/**
 * 商品信息Service业务层处理
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Slf4j
@Service
public class ProjectInfoServiceImpl extends ServiceImpl<ProjectInfoMapper, ProjectInfo>  implements ProjectInfoService {


    @Autowired
    private AsProductAttributeInfoService asProductAttributeInfoService;

    @Autowired
    private ProjectSkuInfoService projectSkuInfoService;

    @Autowired
    private CategoryInfoService categoryInfoService;

    @Autowired
    private RuleAttrInfoService ruleAttrInfoService;

    @Autowired
    private BrandInfoService brandInfoService;

    @Autowired
    private AttributeInfoService attributeInfoService;

    /**
     * 查询商品信息列表
     *
     * @param projectInfo 商品信息
     * @return 商品信息
     */
    @Override
    public List<ProjectInfo> list(ProjectInfo projectInfo) {
        LambdaQueryWrapper<ProjectInfo> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(projectInfo.getName())){
            queryWrapper.like(ProjectInfo::getName, projectInfo.getName());
        }

        if (ObjUtils.notNull(projectInfo.getIntroduction())){
            queryWrapper.eq(ProjectInfo::getIntroduction, projectInfo.getIntroduction());
        }

        if (ObjUtils.notNull(projectInfo.getMianType())){
            queryWrapper.eq(ProjectInfo::getMianType, projectInfo.getMianType());
        }

        if (ObjUtils.notNull(projectInfo.getParentType())){
            queryWrapper.eq(ProjectInfo::getParentType, projectInfo.getParentType());
        }

        if (ObjUtils.notNull(projectInfo.getType())){
            queryWrapper.eq(ProjectInfo::getType, projectInfo.getType());
        }

        if (ObjUtils.notNull(projectInfo.getImage())){
            queryWrapper.eq(ProjectInfo::getImage, projectInfo.getImage());
        }

        if (ObjUtils.notNull(projectInfo.getCarouselImages())){
            queryWrapper.eq(ProjectInfo::getCarouselImages, projectInfo.getCarouselImages());
        }

        if (ObjUtils.notNull(projectInfo.getStatus())){
            queryWrapper.eq(ProjectInfo::getStatus, projectInfo.getStatus());
        }

        if (ObjUtils.notNull(projectInfo.getRuleId())){
            queryWrapper.eq(ProjectInfo::getRuleId, projectInfo.getRuleId());
        }

        if (ObjUtils.notNull(projectInfo.getBrandId())){
            queryWrapper.eq(ProjectInfo::getBrandId, projectInfo.getBrandId());
        }

        return list(queryWrapper);
    }

    /**
     * 保存商品信息
     *
     * @param projectInfoSaveReq 请求对象
     *
     * @return
     */
    @Override
    public boolean save (ProjectInfoSaveReq projectInfoSaveReq) {
        ProjectAddModel projectAddModel = projectInfoSaveReq.getProjectAddModel();
        ProjectInfo projectInfo = ProjectInfo.saveModelBuild(projectAddModel, SecurityUtils::getUsername);
        boolean save = this.save(projectInfo);
        if (save){
            // 属性值
            List<AttrValueModel> attrValueList = projectInfoSaveReq.getAttrValueList();
            List<AsProductAttributeInfo> asProductAttributeInfoList = attrValueList.stream()
                    .map(attrValueModel -> AsProductAttributeInfo.attrValueModelBuild(attrValueModel, projectInfo::getId))
                    .toList();
            if (!asProductAttributeInfoList.isEmpty()){
                asProductAttributeInfoService.saveBatch(asProductAttributeInfoList);
            }
            // sku ProductSkuModel -》 ProjectSkuInfo
            List<ProductSkuModel> productSkuModelList = projectInfoSaveReq.getProductSkuList();
            List<ProjectSkuInfo> projectSkuInfoList = productSkuModelList.stream().map(
                    productSkuModel -> ProjectSkuInfo.productSkuModelBuild(productSkuModel, projectInfo::getId)
            ).toList();
            if (!projectSkuInfoList.isEmpty()){
                projectSkuInfoService.saveBatch(projectSkuInfoList);
            }
        }
        return save;
    }

    /**
     * 通过商品ID获取商品详情
     *
     * @param id 商品ID
     * @return 商品详情
     */
    @Override
    public ProjectDetailResp getDetailInfo (Long id) {
        // 商品信息获取
        ProjectInfo projectInfo = this.getById(id);
        // 品牌信息
        BrandInfo brandInfo = this.brandInfoService.getById(projectInfo.getBrandId());
        // 品类集合
        List<CategoryInfo> categoryInfoList = categoryInfoService.listByIds(new ArrayList<>() {{
            add(projectInfo.getMianType());
            add(projectInfo.getParentType());
            add(projectInfo.getType());
        }});
        // 商品Sku集合
        List<ProjectSkuInfo> projectSkuInfoList = this.projectSkuInfoService.list(new LambdaQueryWrapper<>() {{
            eq(ProjectSkuInfo::getProjectId, id);
        }});

        // 商品和属性集合
        List<AsProductAttributeInfo> productAttributeInfoList = this.asProductAttributeInfoService.list(new LambdaQueryWrapper<>() {{
            eq(AsProductAttributeInfo::getProductId, id);
        }});
        // 商品规格
        List<RuleAttrAddModel> ruleAttrModelList = ruleAttrInfoService.list(new LambdaQueryWrapper<>() {{
                    eq(RuleAttrInfo::getRuleId, projectInfo.getRuleId());
                }}).stream()
                .map(RuleAttrAddModel::infoBuild).toList();


        CategoryCommonElementResp templateAttribute = this.categoryInfoService.getTemplateAttributeByCateGoryId(projectInfo.getType());
        List<TemplateAttributeGroupModel> templateAttributeGroupList = templateAttribute.getTemplateAttributeGroupList();
        List<TemplateAttributeModel> templateAttributeList = new ArrayList<>(){{
            addAll(templateAttribute.getTemplateAttributeList());
        }};
        // 属性组和商品属性的ID
        List<Long> notInAttributeIdList = new ArrayList<>();
        List<Long> attributeGroupIdList = templateAttributeGroupList.stream()
                .flatMap(templateAttributeGroupModel -> templateAttributeGroupModel.getAttributeList().stream())
                .map(TemplateAttributeModel::getId)
                .toList();
        List<Long> attributeIdList = templateAttributeList.stream()
                .map(TemplateAttributeModel::getId)
                .toList();
        if (!attributeGroupIdList.isEmpty()){
            notInAttributeIdList.addAll( attributeGroupIdList );
        }
        if (!attributeIdList.isEmpty()){
            notInAttributeIdList.addAll( attributeIdList );
        }
        // 添加上，商品的自有属性
        List<AsProductAttributeInfo> productAttributeList = this.asProductAttributeInfoService.list(
                new LambdaQueryWrapper<>() {{
                    eq(AsProductAttributeInfo::getProductId, projectInfo.getId());
                    notIn(AsProductAttributeInfo::getAttributeId, notInAttributeIdList);
                }}
        );

        List<TemplateAttributeModel> projectAttributeList = new ArrayList<>();
        if (!productAttributeList.isEmpty()){
            List<Long> attrIdList = productAttributeList.stream()
                    .map(AsProductAttributeInfo::getAttributeId)
                    .toList();
            projectAttributeList = attributeInfoService.list(
                            new LambdaQueryWrapper<>() {{
                                in(AttributeInfo::getId, attrIdList);
                            }}
                    ).stream()
                    .map(TemplateAttributeModel::attributeInfoBuild)
                    .toList();
        }
        // 把自有属性添加到商品属性的集合当中，进行合并
        if (!projectAttributeList.isEmpty()){
            templateAttributeList.addAll(projectAttributeList);
        }
        return ProjectDetailResp.builder()
                .projectInfo(projectInfo)
                .brandInfo(brandInfo)
                .categoryInfoList(categoryInfoList)
                .projectSkuInfoList(projectSkuInfoList)
                .productAttributeInfoList(productAttributeInfoList)
                .ruleAttrModelList(ruleAttrModelList)
                .attributeInfoList(templateAttributeList)
                .attributeGroupList(templateAttributeGroupList)
                .build();
    }

    @Override
    public Result UpdProjectInfo(Long id, ProjectInfoEditReq projectInfoEditReq) {
        // 调用 updateById 方法，修改商品基本信息
        this.updateById(ProjectAddModel.editBuild(id,projectInfoEditReq));
        //检查属性列表是否为空
        if(!com.alibaba.nacos.common.utils.CollectionUtils.isEmpty(projectInfoEditReq.getAttrValueList())){
            //拿到属性值不为空的 构建 AttributeInfo 对象列表
            List<AttributeInfo> attributeInfos = projectInfoEditReq.getAttrValueList().stream().filter(projectModel -> !StringUtils.isBlank(projectModel.getValue())).map(item->{
                AttributeInfo attributeInfo =AttributeInfo.builder().id(item.getId()).name(item.getValue()).build();
                return attributeInfo;
            }).toList();
            //批量修改属性信息
            attributeInfoService.updateBatchById(attributeInfos);
        }
        //检查产品的SKU列表是否为空
        if(!CollectionUtils.isEmpty(projectInfoEditReq.getProductSkuList())){
            //构建 ProjectSkuInfo 对象列表
            List<ProjectSkuInfo> skuInfoList = projectInfoEditReq.getProductSkuList().stream().map(projectSkuInfoModel-> {
                ProjectSkuInfo projectSkuInfo = ProjectSkuInfo.builder().sku(projectSkuInfoModel.getSku())
                        .image(projectSkuInfoModel.getImage())
                        .stock(projectSkuInfoModel.getStock())
                        .price(projectSkuInfoModel.getPrice())
                        .build();
                return projectSkuInfo;
            }).toList();
            //循环遍历 SKU 列表 循环修改
            skuInfoList.forEach(projectSkuInfoModel->{
                LambdaUpdateWrapper<ProjectSkuInfo> wrapper = new LambdaUpdateWrapper<>();
                //如果图片不为空，则修改图片字段
                if(!StringUtils.isBlank(projectSkuInfoModel.getImage())){
                    wrapper.set(ProjectSkuInfo::getImage,projectSkuInfoModel.getImage());
                }
                //修改库存和价格字段
                wrapper.set(ProjectSkuInfo::getStock,projectSkuInfoModel.getStock());
                wrapper.set(ProjectSkuInfo::getPrice,projectSkuInfoModel.getPrice());
                //修改条件 为SKU字段
                wrapper.eq(ProjectSkuInfo::getSku,projectSkuInfoModel.getSku());
                //执行更新操作
                projectSkuInfoService.update(null,wrapper);
            });
        }
        return Result.success(null,"修改");
    }

//    @Override
//    public Result UpdProjectInfo(Long id, ProjectInfoEditReq projectInfoEditReq) {
//        this.updateById(ProjectAddModel.editBuild(id,projectInfoEditReq));
//        if(!CollectionUtils.isEmpty(projectInfoEditReq.getAttrValueList())){
//            List<AttributeInfo>  attributeInfoList= projectInfoEditReq.getAttrValueList().stream().filter(attributeInfoUpdModel->!StringUtils.isBlank(attributeInfoUpdModel.getValue()))
//                    .map(item->{
//                        AttributeInfo attributeInfo = AttributeInfo.builder().id(item.getId()).name(item.getValue()).build();
//                        return attributeInfo;
//                    }).toList();
//            attributeInfoService.updateBatchById(attributeInfoList);
//        }
//
//        if(!CollectionUtils.isEmpty(projectInfoEditReq.getProductSkuList())){
//            List<ProjectSkuInfo> skuInfoList =projectInfoEditReq.getProductSkuList().stream().map(projectSKUModel -> {
//                ProjectSkuInfo projectSkuInfo = ProjectSkuInfo.builder()
//                        .sku(projectSKUModel.getSku())
//                        .image(projectSKUModel.getImage())
//                        .stock(projectSKUModel.getStock())
//                        .price(projectSKUModel.getPrice())
//                        .build();
//                return projectSkuInfo;
//            }).toList();
//            skuInfoList.forEach(projectSkuInfo -> {
//                LambdaUpdateWrapper<ProjectSkuInfo> wrapper = new LambdaUpdateWrapper<>();
//                if(!StringUtils.isBlank(projectSkuInfo.getImage())){
//                    wrapper.set(ProjectSkuInfo::getImage,projectSkuInfo.getImage());
//                }
//                wrapper.set(ProjectSkuInfo::getStock,projectSkuInfo.getStock());
//                wrapper.set(ProjectSkuInfo::getPrice,projectSkuInfo.getPrice());
//                wrapper.eq(ProjectSkuInfo::getSku, projectSkuInfo.getSku());
//                projectSkuInfoService.update(null,wrapper);
//            });
//
//        }
//        return Result.success("修改成功");
//    }




}
